;
; sicmenu.asm
;
; SIC! Menu
;
; date: 2011.04.17
; last modified: 2011.09.14
;

buf_buffer	equ $700
SIC_Code	equ $800

sic_dlist	equ $f00
sic_screen	equ $2000
sic_logo	equ $1000

MEMTOP		equ $2E5
MEMLO		equ $2E7

WARMST		equ $08
BOOT?		equ $09
DOSINI		equ $0C

KBCODES		equ $2FC

dmactl		equ $d400
wsync		equ $d40a
nmien		equ $d40e
;
	opt h-
	opt ?+
	org $a000

.local ,SIC_Code

sic_restore	
	lda #0
	sta $d500
	jmp $E477

boot_load

	lda #0
	sta WARMST
	
	lda <sic_restore
	sta DOSINI
	lda >sic_restore
	sta DOSINI+1

        ldy <loader_end
        sty MEMLO
        lda >loader_end
        sta MEMLO+1

	ldx  #$FF
	txs
	
; na poczatek czyszczenie pamieci od MEMLO (A,Y) do MEMTOP

	STA   ?InMemClearLoop+2
?OutMemClearLoop
	LDA  #$00
?InMemClearLoop
	STA   $0900,Y
	INY
	BNE   ?InMemClearLoop
	INC   ?InMemClearLoop+2
	LDA   ?InMemClearLoop+2
	CMP   MEMTOP+1
	BCC   ?OutMemClearLoop
	LDA   MEMTOP+1
	STA   ?LastMemPageClear+2
	LDY   MEMTOP
	LDA  #$00
?LastMemPageClear
	STA   $8000,Y
	DEY
	CPY  #$FF
	BNE   ?LastMemPageClear
	

?set_x	ldx #0		; values for X,Y are set at file selection 	
?set_y	ldy #0
	
	lda #$FF
	sta KBCODES
	sta runflg
	bne SIC_Loader2	; bra
	
;--------------------------------------
; SIC_Loader
;
; Input
;  X - offset (HI Byte)
;  Y - bank
;--------------------------------------
SIC_Loader
	lda #$FF	; entry with filesize set to $FFFFFF
	sta filesize
	sta filesize+1
	sta filesize+2

SIC_Loader2		; entry with a custom filesize

	sta $2e1	; A is still $FF
	sta eofflg	; not eof

	jsr InitBuf
	
	lda #0
	sta $2e0
	sta $2e1

GetBlockAdr

	lda #$d0
	sta $2e3

	jsr GetByte
	sta l_adr
	jsr GetByte
	sta l_adr+1

	bne GetBlockAdr_L1
	
	ldx l_adr
	beq End		; $0000 ?

GetBlockAdr_L1
	cmp #$ff
	bne GetEndAdr
	lda l_adr
	cmp #$ff
	beq GetBlockAdr
	cmp #$fe
	beq End

GetEndAdr
	jsr GetByte
	sta l_adr2
	jsr GetByte
	sta l_adr2+1
	
	bne SetBlkAdr
	ldx l_adr2
		
	bne SetBlkAdr
	
End
	lda $02e0
	sta _run+1
	lda $02e1
	sta _run+2
	ora _run+1
	bne _run	
xrts	rts

_run	jmp sic_restore


SetBlkAdr

?check_range		; check if the block overwrites the loader

	ldx l_adr+1
	cpx MEMLO+1
	bne ?cmp2	;l_adr+1 > memlo+1
	lda l_adr
	cmp MEMLO	;gdy starsze bajty równe, porównaj młodsze
?cmp2	bcs ?in_range	;bo ładujemy powyżej memlo 

	cpx #$07	;czy ładujemy poniżej $700 ? 
	bcs ?out_range	;nie,to wypad
	
	lda l_adr2+1	;gdy ładujemy poniżej $700, sprawdzamy końcowy adres 
	cmp #$07	;gdy całosć poniżej $700, to ok
	bcc ?in_range
	
?out_range
	jmp sic_restore
	
?in_range

	inc l_adr2
	bne *+5
	inc l_adr2+1

	ldx l_adr
	lda l_adr+1
	sta _p+2

GetBlock
	jsr GetByte
	beq End
_p	sta $100,x
	inx
	bne *+5
	inc _p+2

	cpx l_adr2
	bne GetBlock
	lda _p+2
	cmp l_adr2+1
	bcc GetBlock

InitBlock
;	lda l_adr+1
;	cmp #$02
;	bne GetBlockAdr
;	lda l_adr
;	cmp #$e2
;	bne GetBlockAdr

	lda runflg	; force run ? (user files only)
	beq ?caroff
	
	lda $2e0	; is run address already set ?
	ora $2e1
	bne ?caroff

	ldx #1	
?setrun	lda l_adr,x	; if not set, then put address of the first block there
	sta $2e0,x
	dex
	bpl ?setrun		

?caroff	jsr CartOff	; in case it has been accidentally turned on (Boulder Dash) 

	lda $2e3
	cmp #$d0
	jeq GetBlockAdr

;
	lda $02e2
	sta _ini+1
	lda $02e3
	sta _ini+2

_ini	jsr xrts

	lda #$d0
	sta $2e3
	jmp GetBlockAdr

l_adr	dta a(0)
l_adr2	dta a(0)

buf_count
	dta 0

buf_bank
	dta 0

filesize
	.byte $FF,$FF,$FF
	
eofflg	.byte $FF

runflg	.byte $0	; force run flag	

;
; InitBuf
;
; Input
;  X - offset (HI byte)
;  Y - bank
;
InitBuf
	clc
	txa
	adc #$80
	sta buf_p+1

	sty buf_bank

	ldy #255
	jmp GetByte_L1

;
; CartOff
;
CartOff
	
	lda #%01000000
	sta $d500
	sta $d40a
	sta $d40a
	lda $d013
	sta $3fa
	
;?resdma	lda #$22
;	sta dmactl
?resnmi	lda #$40
	sta nmien
	
	rts

;
; MapBank
;
; Input:
;  ACC - bank
;
MapBank
	ora #%00100000
	pha
	
	stx ?savx+1

?flick	ldx $D40B
	cpx #$70
	bne ?flick	
	
	;ldx $22f
	;stx CartOff.?resdma+1
	ldx nmien
	stx CartOff.?resnmi+1
	ldx #0
	;stx dmactl
	stx nmien
			
	sta $d500
	sta $d40a		;WSYNC
	sta $d40a		;WSYNC
	lda $d013		;TRIG3
	sta $3fa		;GINTLK
	
?savx	ldx #0
	pla
	rts

;
; GetByte (A = byte, Z = 1 on error)
;
GetByte

	lda eofflg
	beq XGetByt	
	
	lda #0
	sta eofflg
	
	ldy #-3
	clc
?decfs	lda filesize-$0100+3,y
	sbc #$00
	sta filesize-$0100+3,y
	ora eofflg
	sta eofflg	
	iny
	bne ?decfs

	ldy buf_count
	lda buf_buffer,y

GetByte_L1
	iny
	sty buf_count
	seq
	
XGetByt	rts

; Fill Buffer
	pha

	lda buf_bank
	jsr MapBank

FillBuffer
buf_p	equ *+1
	lda $1000,y
	sta buf_buffer,y
	iny
	bne FillBuffer

	inc buf_p+1

	lda buf_p+1
	cmp #$c0
	bmi GetByte_Skip

	lda #$80
	sta buf_p+1
	inc buf_bank

GetByte_Skip
	jsr CartOff

	pla
	
	ldy #1		; success
	rts
	
loader_end
	
.print "LOADER ENDS=",*
     
;--------------------------------------
; SIC_InitMenu
;--------------------------------------
SIC_InitMenu
	lda #<SIC_Menu
	sta $0a
	lda #>SIC_Menu
	sta $0b

	lda #<BlankDList
	sta 560
	lda #>BlankDList
	sta 561

	lda #0
	jsr MapBank
	ldx $8000
	ldy $8001
	beq SIC_Menu

; load intro
	lda $d40b
	bne *-3

	lda #0
	sta 559
	sta $d400

	jsr SIC_Loader

;--------------------------------------
; SIC_Menu
;--------------------------------------
zp		equ $80
scrp		equ $82
scrp2		equ $84
adr		equ $86

listp		equ $88
selectp		equ $89
select_scrp	equ $8a
tmp		equ $8c

;
SIC_Menu
	lda #<SIC_Menu
	sta $0a
	lda #>SIC_Menu
	sta $0b

	lda #0
	sta listp
	sta selectp

	lda #1
	sta $3f8

	lda $d301
	ora #2
	sta $d301	; disable BASIC

;
; build display list
;
	lda #<sic_dlist
	sta zp
	lda #>sic_dlist
	sta zp+1

	ldy #0

	lda #$70
	sta (zp),y
	iny
	sta (zp),y
	iny
	sta (zp),y
	iny

	lda #0
	jsr MapBank

	ldx $8011
	beq NoLogo

	lda #$4e
	sta (zp),y
	iny

	lda #0
	sta (zp),y
	iny
	lda #>sic_logo
	sta (zp),y
	iny
 	dex

L1
	lda #$0e
	sta (zp),y
	iny
	dex
	bne L1

NoLogo
	lda #$40
	ora $8028
	sta (zp),y
	iny
	lda #<sic_screen
	sta (zp),y
	iny
	lda #>sic_screen
	sta (zp),y
	iny

	ldx $8010		; number of lines for menu
	dex

L2
	lda $8028
	sta (zp),y
	iny
	dex
	bne L2

	lda #$41
	sta (zp),y
	iny
	lda zp
	sta (zp),y
	iny
	lda zp+1
	sta (zp),y

; load logo data
	ldx $8011
	beq LoadLogo_Skip

	ldx $8012
	ldy $8013
	jsr SIC_Loader

LoadLogo_Skip

;
; init music
;
	lda #0
	jsr MapBank

	ldx $8002
	ldy $8003
	beq NoMusic

	jsr SIC_Loader		; load player

	lda #0
	jsr MapBank
	ldx $8004
	ldy $8005
	beq NoMusic

	jsr SIC_Loader		; load music

	lda #0
	jsr MapBank
	lda $8006
	sta _msxini+1
	lda $8007
	sta _msxini+2

	lda $8008
	ldx $8009
	ldy $800a
_msxini	jsr $100

	ldx $800b
	ldy $800c
	stx _msxply+1
	sty _msxply+2

	lda $4000
	cmp #'R'
	bne NotRMT
	lda $4001
	cmp #'M'
	bne NotRMT
	lda $4002
	cmp #'T'
	bne NotRMT

	lda $4006
	cmp #2
	bcc NotRMT

	stx _msxply2+1
	sty _msxply2+2

NotRMT

;
NoMusic

; set display list
	lda #<sic_dlist
	sta 560
	sta $d402
	lda #>sic_dlist
	sta 561
	sta $d403

	lda #0
	jsr MapBank

	lda $801f
	sta $d409

	lda $8024
	sta 712
	sta $d01a

;
; Menu
;
Menu
   
;
ShowFileList
	lda #0
	sta selectp

	jsr ClearScreen

RefreshFileList

	lda #<sic_screen
	sta scrp
	sta scrp2
	lda #>sic_screen
	sta scrp+1
	sta scrp2+1


	ldx $802a
	beq TopMargin_Skip

TopMargin
	jsr NextLine
	dex
	bne TopMargin

TopMargin_Skip

; read file list
;
	lda #0	
	jsr MapBank
	lda #0
	sta adr+1
	lda listp
	asl @
	rol adr+1
	asl @
	rol adr+1
	asl @
	rol adr+1
	asl @
	rol adr+1
	asl @
	rol adr+1

	clc
	adc #64+2
	sta adr
	lda adr+1
	adc #$80
	sta adr+1

	ldx #0

FileList
	ldy #0
	lda (adr),y
	beq FileList_Done

	lda $8029

	clc
	adc scrp
	sta scrp
	bcc *+4
	inc scrp+1

	cpx selectp
	bne FileList_L1

	lda scrp
	sta select_scrp
	lda scrp+1
	sta select_scrp+1

FileList_L1
	txa
	clc
	adc #$21
	jsr PutChar
	lda #0
	jsr PutChar

	lda adr
	ldy adr+1
	jsr PutString

	cpx selectp
	bne FileList_L2

	ldy #26
CopySelect
	lda (select_scrp),y
	sta $600,y
	dey
	bpl CopySelect

FileList_L2
	jsr NextLine

	lda adr
	clc
	adc #32
	sta adr
	bcc *+4
	inc adr+1

	inx
	cpx $8018		; number files per page
	bcc FileList

FileList_Done
	lda #$ff
	sta $2f2
	sta $2fc

;
WaitKey
	lda $d40b
	cmp #$70
	bcc WaitKey

	lda #0
	jsr MapBank

; set logo colors
	lda $8025
	sta $d016
	sta 708
	lda $8026
	sta $d017
	sta 709
	lda $8027
	sta $d018
	sta 710

	lda #34
	sta 559
	sta $d400

_msxply	jsr Void

;
	lda $8011
	lsr @
	clc
	adc #15
	cmp #$50
	bcc Sync_L1

Sync1
	lda $d40b
	cmp #$40
	bcc Sync1
	jsr _msxply2

Sync_L1
	lda $8011
	lsr @
	clc
	adc #15

Wait2
	cmp $d40b
	bne Wait2

	sta $d40a

; set menu colors
	lda $8020
	sta $d017
	lda $8021
	sta $d018
	lda $8022
	sta $d019
	lda $8023
	sta $d016

;
	lda $8011
	lsr @
	clc
	adc #15
	cmp #$50
	bcs Sync_L2

Sync2
	lda $d40b
	cmp #$40
	bcc Sync2
	jsr _msxply2

Sync_L2
;
	ldy #26

_c2	lda #0
	inc _c2+1
	lsr @
	lsr @
	lsr @
	lsr @
	and #1
	bne FlashSelect2

	lda #0

FlashSelect
	sta (select_scrp),y
	dey
	bpl FlashSelect
	jmp FlashSelect_End

FlashSelect2
	lda $600,y
	sta (select_scrp),y
	dey
	bpl FlashSelect2

FlashSelect_End

_c	lda #0
	inc _c+1
	and #7
	bne Joy_L0

Joy
	lda $d300
	and #%00000010
	bne Joy_L1

	clc
	lda selectp
	adc #1
	cmp $8018
	bcc *+4
	lda #0

	sta selectp
	jeq NextPage

	clc
	adc listp
	cmp $8019
	jcc RefreshFileList	;ShowFileList

	lda #0
	sta selectp
	sta listp
	jmp RefreshFileList	;ShowFileList

Joy_L1
	lda $d300
	and #%00000001
	bne Joy_L0

	sec
	lda selectp
	sbc #1
	bpl Joy_L1a
	lda $8018
	sec
	sbc #1
	sta selectp

	sec
	lda listp
	sbc $8018		; number files per page
	bpl *+6
	lda #0
	sta selectp

	sta listp
	jmp RefreshFileList	;ShowFileList

Joy_L1a
	sta selectp
	jmp RefreshFileList	;ShowFileList

Joy_L0
	lda selectp
	ldx $d010
	jeq LoadFile

Joy_L0a

	lda $2fc
	cmp #$ff
	jeq WaitKey

	cmp #33
	bne CheckKey

; SPACE - next page
NextPage
	clc
	lda listp
	adc $8018		; number files per page
	cmp $8019		; total number of files
	bcc *+4
	lda #0
	sta listp
	jmp ShowFileList

;
CheckKey
	cmp #64
	jcs WaitKey

	tax
;	lda #0
;	sta zp+1

	lda keytable,x
	bpl LoadFile
	sta $2fc
	jmp WaitKey

LoadFile
	pha

	ldx #0
	stx zp+1
	txa

SndOff
	sta $d200,x
	sta $d210,x
	inx
	cpx #9
	bcc SndOff

	pla
	clc
	adc listp
	asl @
	rol zp+1
	asl @
	rol zp+1
	asl @
	rol zp+1
	asl @
	rol zp+1
	asl @
	rol zp+1
	sta zp

	clc
	lda zp
	adc #64
	sta zp
	lda zp+1
	adc #$80
	sta zp+1

	ldy #0
	lda (zp),y
	sta boot_load.?set_x+1
	iny
	lda (zp),y
	jeq ShowFileList
	sta boot_load.?set_y+1

	ldy #29			; get file size 
?setfs	lda (zp),y
	sta filesize-29,y
	iny
	cpy #32
	bmi ?setfs
	

;
; clear memory
;

	jsr Cartoff
	
     	ldx #$00
	txa
	
; wstepne czyszczenie (reszte RAM czysci procedura ladujaca - dzieki czemu czysci tez program glowny)
?ClearLoop1
	STA   $0100,X         ; STOS !!!
	STA   $0400,X        ; bufor magnetofonu (128) i obszar zarezerwowany?? (drugie 128b)
	STA   $0500,X 
	STA   $0600,X 
	CPX  #$80             ;tylko ponad $80
	BCC   ?NoZpage
	STA   $00,X           ; czyli polowa strony zerowej
?NoZpage
	INX
	BNE   ?ClearLoop1
	

	lda <boot_load
	sta DOSINI
	lda >boot_load
	sta DOSINI+1
	
	lda #1
	sta BOOT?
	
	jmp $E474	; warm boot
;
;	jsr SIC_Loader
;	jmp SIC_Menu

;
; _msxply2
;
_msxply2
	jmp Void

;
; keytable
;
keytable
	dta $0b,$09,$ff,$ff,$ff,$0a,$ff,$ff ; $00-$07
	dta $0e,$ff,$0f,$14,$ff,$08,$ff,$ff ; $08-$0f
	dta $15,$ff,$02,$ff,$ff,$01,$17,$ff ; $10-$17
	dta $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff ; $18-$1f
	dta $ff,$ff,$ff,$0d,$ff,$0c,$ff,$ff ; $20-$27
	dta $11,$ff,$04,$ff,$ff,$13,$16,$10 ; $28-$2f
	dta $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff ; $30-$37
	dta $05,$07,$03,$ff,$ff,$06,$12,$00 ; $38-$3f

;
; PutString
;
PutString
	sta zp
	sty zp+1

	ldy #0

PutString_Loop
	lda (zp),y
	bmi PutString_End
	sta (scrp),y
	iny
	bne PutString_Loop

PutString_End
	tya
	clc
	adc scrp
	sta scrp
	bcc *+4
	inc scrp+1
	rts

;
; PutChar
;
PutChar
	ldy #0
	sta (scrp),y
	iny
	jmp PutString_End

;
; NextLine
;
NextLine
	clc
	lda scrp2
	adc #40
	sta scrp2
	sta scrp
	lda scrp2+1
	adc #0
	sta scrp2+1
	sta scrp+1
	rts

;
; ClrScr
;
ClearScreen
	ldx #0
	txa

ClrScr
	lda $9800,x
	sta sic_screen,x
	lda $9900,x
	sta sic_screen+256,x
	lda $9a00,x
	sta sic_screen+512,x
	lda $9b00,x
	sta sic_screen+768,x
	inx
	bne ClrScr

	rts

;
; Void
;
Void
	rts

BlankDList
	dta $41,a(BlankDList)


.print "SICMENU ENDS=",*
.endl

;--------------------------------------
; SIC_Init
;--------------------------------------
SIC_Init
.local
	lda #1
	sta $3f8
	lda $d301
	ora #2
	sta $d301	; disable BASIC

	ldx #0
Move
	lda $a000,x
	sta SIC_Code,x
	lda $a100,x
	sta SIC_Code+$100,x
	lda $a200,x
	sta SIC_Code+$200,x
	lda $a300,x
	sta SIC_Code+$300,x
	lda $a400,x
	sta SIC_Code+$400,x
	lda $a500,x
	sta SIC_Code+$500,x
	lda $a600,x
	sta SIC_Code+$600,x
	inx
	bne Move

	jmp SIC_InitMenu
.endl

;--------------------------------------
; RTS
;--------------------------------------
_Rts
	rts

;
; cart alignment
;
.rept ($a000 + 8192 - 6 - *)
	dta 0
.endr

;
; cart init info
;
	dta a(SIC_Init)
	dta 0
	dta 4
	dta a(_Rts)
